@@ -5,9 +5,15 @@
 #' This function tries to find the optimum average series between a group of time series in DTW space. Refer to
 #' the cited article for specific details on the algorithm.
 #'
-#' If a given series reference is provided in \code{center}, the algorithm should always converge to the same
+#' If a given series reference is provided in \code{centroid}, the algorithm should always converge to the same
 #' result provided the elements of \code{X} keep the same values, although their order may change.
 #'
+#' The windowing constraint uses a centered window. The calculations expect a value in \code{window.size}
+#' that represents the distance between the point considered and one of the edges of the window. Therefore,
+#' if, for example, \code{window.size = 10}, the warping for an observation \eqn{x_i} considers the points
+#' between \eqn{x_{i-10}} and \eqn{x_{i+10}}, resulting in \code{10(2) + 1 = 21} observations falling within
+#' the window.
+#'
 #' @section Parallel Computing:
 #'
 #' Please note that running tasks in parallel does \strong{not} guarantee faster computations.
@@ -61,37 +67,48 @@
 #' @param X A data matrix where each row is a time series, or a list where each element is a time series.
 #' Multivariate series should be provided as a list of matrices where time spans the rows and the variables
 #' span the columns.
-#' @param center Optionally, a time series to use as reference. Defaults to a random series of \code{X} if
+#' @param centroid Optionally, a time series to use as reference. Defaults to a random series of \code{X} if
 #' \code{NULL}. For multivariate series, this should be a matrix with the same characteristics as the
 #' matrices in \code{X}.
+#' @param center Deprecated, please use \code{centroid} instead.
 #' @param max.iter Maximum number of iterations allowed.
 #' @param norm Norm for the local cost matrix of DTW. Either "L1" for Manhattan distance or "L2" for Euclidean
 #' distance.
 #' @param window.size Window constraint for the DTW calculations. \code{NULL} means no constraint. A slanted
 #' band is used by default.
-#' @param delta At iteration \code{i}, if \code{all(abs(center_{i}} \code{ - center_{i-1})} \code{ < delta)},
+#' @param delta At iteration \code{i}, if \code{all(abs(centroid_{i}} \code{ - centroid_{i-1})} \code{ < delta)},
 #' convergence is assumed.
 #' @param error.check Should inconsistencies in the data be checked?
 #' @param trace If \code{TRUE}, the current iteration is printed to screen.
-#' @param ... Further arguments for \code{\link[dtw]{dtw}}, e.g. \code{step.pattern}.
+#' @param ... Further arguments for \code{\link[dtw]{dtw}} or \code{\link{dtw_basic}}, e.g.
+#' \code{step.pattern}.
+#' @param dba.alignment Character indicating which function to use for calculating alignments, either
+#' \code{\link[dtw]{dtw}} or \code{\link{dtw_basic}}. The latter should be faster.
 #'
 #' @return The average time series.
 #'
 #' @export
 #'
 
-DBA <- function(X, center = NULL, max.iter = 20L,
+DBA <- function(X, centroid = NULL, center = NULL, max.iter = 20L,
                 norm = "L1", window.size = NULL, delta = 1e-3,
-                error.check = TRUE, trace = FALSE, ...) {
+                error.check = TRUE, trace = FALSE, ..., dba.alignment = "dtw_basic") {
+     if (!missing(center)) {
+          warning("The 'center' argument has been deprecated, please use 'centroid' instead.")
+
+          if (is.null(centroid)) centroid <- center
+     }
+
+     dba.alignment <- match.arg(dba.alignment, c("dtw", "dtw_basic"))
 
      X <- consistency_check(X, "tsmat")
 
-     if (is.null(center))
-          center <- X[[sample(n, 1L)]] # Random choice
+     if (is.null(centroid))
+          centroid <- X[[sample(length(X), 1L)]] # Random choice
 
      if (error.check) {
           consistency_check(X, "vltslist")
-          consistency_check(center, "ts")
+          consistency_check(centroid, "ts")
      }
 
      norm <- match.arg(norm, c("L1", "L2"))
@@ -108,30 +125,12 @@ DBA <- function(X, center = NULL, max.iter = 20L,
           w <- NULL
      }
 
-     ## check for multivariate case
-     dims <- sapply(X, function(x) {
-          if (is.null(dim(x)))
-               0L
-          else
-               ncol(x)
-     })
-
-     if (length(unique(dims)) != 1L)
-          stop("Inconsistent dimensions across series.")
-
-     if (any(dims > 0L)) {
+     ## utils.R
+     if (check_multivariate(X)) {
           ## multivariate
-          ncols <- ncol(X[[1L]])
-          ncols <- rep(1L:ncols, length(X))
+          mv <- reshape_multviariate(X, centroid) # utils.R
 
-          x <- do.call(cbind, X)
-          x <- split.data.frame(t(x), ncols)
-
-          c <- lapply(1L:ncol(center), function(idc) {
-               center[ , idc, drop = TRUE]
-          })
-
-          new_c <- mapply(x, c, SIMPLIFY = FALSE,
+          new_c <- mapply(mv$series, mv$cent, SIMPLIFY = FALSE,
                           FUN = function(xx, cc) {
                                DBA(xx, cc,
                                    norm = norm,
@@ -140,19 +139,18 @@ DBA <- function(X, center = NULL, max.iter = 20L,
                                    delta = delta,
                                    error.check = FALSE,
                                    trace = trace,
+                                   dba.alignment = dba.alignment,
                                    ...)
                           })
 
           return(do.call(cbind, new_c))
      }
 
-     ## for C helper
-     square <- norm == "L2"
-
-     n <- length(X)
-
      ## pre-allocate local cost matrices
-     LCM <- lapply(X, function(x) { matrix(0, length(x), length(center)) })
+     if (dba.alignment == "dtw")
+          LCM <- lapply(X, function(x) { matrix(0, length(x), length(centroid)) })
+     else
+          LCM <- lapply(X, function(x) { matrix(0, length(x) + 1L, length(centroid) + 1L) })
 
      ## maximum length of considered series
      L <- max(lengths(X))
@@ -165,21 +163,31 @@ DBA <- function(X, center = NULL, max.iter = 20L,
 
      ## Iterations
      iter <- 1L
-     center_old <- center
+     centroid_old <- centroid
+
+     if (trace) cat("\tDBA Iteration:")
 
      while(iter <= max.iter) {
-          ## Return the coordinates of each series in X grouped by the coordinate they match to in the center time series
+          ## Return the coordinates of each series in X grouped by the coordinate they match to in the centroid time series
           ## Also return the number of coordinates used in each case (for averaging below)
           xg <- foreach(X = Xs, LCM = LCMs,
                         .combine = c,
                         .multicombine = TRUE,
+                        .export = "enlist",
                         .packages = c("dtwclust", "stats")) %dopar% {
                              mapply(X, LCM, SIMPLIFY = FALSE, FUN = function(x, lcm) {
-                                  .Call("update_lcm", lcm, x, center, square, PACKAGE = "dtwclust")
+                                  if (dba.alignment == "dtw") {
+                                       .Call("update_lcm", lcm, x, centroid,
+                                             isTRUE(norm == "L2"), PACKAGE = "dtwclust")
 
-                                  d <- do.call(dtw::dtw, c(list(x = lcm,
-                                                                window.size = w),
-                                                           dots))
+                                       d <- do.call(dtw::dtw, enlist(x = lcm, window.size = w, dots = dots))
+
+                                  } else {
+                                       d <- do.call(dtw_basic, enlist(x = x, y = centroid,
+                                                                      window.size = w, norm = norm,
+                                                                      backtrack = TRUE, gcm = lcm,
+                                                                      dots = dots))
+                                  }
 
                                   x.sub <- stats::aggregate(x[d$index1],
                                                             by = list(ind = d$index2),
@@ -196,30 +204,32 @@ DBA <- function(X, center = NULL, max.iter = 20L,
           ## Put everything in one big data frame
           xg <- reshape2::melt(xg)
 
-          ## Aggregate according to index of center time series (Var1) and also the variable type (Var2)
+          ## Aggregate according to index of centroid time series (Var1) and also the variable type (Var2)
           xg <- stats::aggregate(xg$value, by = list(xg$Var1, xg$Var2), sum)
 
           ## Average
-          center <- xg$x[xg$Group.2 == "sum"] / xg$x[xg$Group.2 == "n"]
+          centroid <- xg$x[xg$Group.2 == "sum"] / xg$x[xg$Group.2 == "n"]
 
-          if (all(abs(center - center_old) < delta)) {
+          if (all(abs(centroid - centroid_old) < delta)) {
                if (trace)
-                    cat("DBA: Iteration", iter ,"- Converged!\n\n")
+                    cat("", iter ,"- Converged!\n")
 
                break
 
           } else {
-               center_old <- center
+               centroid_old <- centroid
 
-               if (trace)
-                    cat("DBA: Iteration", iter, "\n")
+               if (trace) {
+                    cat(" ", iter, ",", sep = "")
+                    if (iter %% 10 == 0) cat("\n\t\t")
+               }
 
                iter <- iter + 1L
           }
      }
 
      if (iter > max.iter && trace)
-          warning("DBA algorithm did not 'converge' within the allowed iterations.")
+          cat(" Did not 'converge'\n")
 
-     as.numeric(center)
+     as.numeric(centroid)
 }
