@@ -3,7 +3,6 @@
 # ========================================================================================================
 
 ddist <- function(distance, control, distmat) {
-
      needs_window <- c("dtw_lb", "lbk", "lbi")
 
      if (distance %in% needs_window)
@@ -16,19 +15,23 @@ ddist <- function(distance, control, distmat) {
 
      ## Closures will capture the values of the constants
 
-     distfun <- function(x, centers = NULL, ...) {
+     distfun <- function(x, centroids = NULL, ...) {
+          if (!is.null(list(...)$centers)) {
+               warning("The 'centers' argument has been deprecated, please use 'centroids' instead.")
+
+               if (is.null(centroids)) centroids <- list(...)$centers
+          }
 
           if (!is.null(distmat)) {
-
-               if (is.null(centers)) {
+               if (is.null(centroids)) {
                     d <- distmat
 
                } else {
                     ## distmat matrix already calculated, just subset it
-                    id_XC <- attr(centers, "id_cent")
+                    id_XC <- attr(centroids, "id_cent")
 
                     if (is.null(id_XC) || any(is.na(id_XC)))
-                         id_XC <- sapply(centers, FUN = function(i.c) {
+                         id_XC <- sapply(centroids, FUN = function(i.c) {
                               i.row <- sapply(x, function(i.x) {
                                    if (length(i.x) == length(i.c))
                                         ret <- all(i.x == i.c)
@@ -80,9 +83,9 @@ ddist <- function(distance, control, distmat) {
                check_parallel()
 
                ## variables/functions from the parent environment that should be exported
-               export <- c("distance", "consistency_check")
+               export <- c("distance", "consistency_check", "enlist")
 
-               if (is.null(centers) && control@symmetric && dist_entry$loop) {
+               if (is.null(centroids) && control@symmetric && dist_entry$loop) {
                     ## WHOLE SYMMETRIC DISTMAT
                     ## Only half of it is computed
 
@@ -104,10 +107,10 @@ ddist <- function(distance, control, distmat) {
 
                                       ## 'dots' has all extra arguments that are valid
                                       dd <- do.call(proxy::dist,
-                                                    c(dots,
-                                                      list(x = x[pairs[ , 1L]],
+                                                    enlist(x = x[pairs[ , 1L]],
                                                            y = x[pairs[ , 2L]],
-                                                           method = distance)))
+                                                           method = distance,
+                                                           dots = dots))
 
                                       dd
                                  }
@@ -126,26 +129,31 @@ ddist <- function(distance, control, distmat) {
                } else {
                     ## WHOLE OR SUBDISTMAT, NOT SYMMETRIC OR loop = FALSE
 
-                    if (is.null(centers))
-                         centers <- x
+                    if (is.null(centroids)) {
+                         centroids <- x
 
-                    dim_names <- list(names(x), names(centers))
+                         ## for dtw_basic_proxy
+                         if (!check_parallel() && has_dots && (is.null(control@window.size) || !check_lengths(x)))
+                              dots$symmetric <- TRUE
+                    }
+
+                    dim_names <- list(names(x), names(centroids))
 
                     if (!is.null(dots$pairwise) && dots$pairwise) {
-                         if (length(x) != length(centers))
+                         if (length(x) != length(centroids))
                               stop("Both sets of data must have the same amount of series for pairwise calculation")
 
-                         centers <- split_parallel(centers)
+                         centroids <- split_parallel(centroids)
                          combine <- c
 
                     } else {
-                         centers <- lapply(1:length(x), function(dummy) centers)
+                         centroids <- lapply(1L:foreach::getDoParWorkers(), function(dummy) centroids)
                          combine <- rbind
                     }
 
                     x <- split_parallel(x)
 
-                    d <- foreach(x = x, centers = centers,
+                    d <- foreach(x = x, centroids = centroids,
                                  .combine = combine,
                                  .multicombine = TRUE,
                                  .packages = control@packages,
@@ -156,10 +164,10 @@ ddist <- function(distance, control, distmat) {
 
                                       ## 'dots' has all extra arguments that are valid
                                       dd <- do.call(proxy::dist,
-                                                    c(dots,
-                                                      list(x = x,
-                                                           y = centers,
-                                                           method = distance)))
+                                                    enlist(x = x,
+                                                           y = centroids,
+                                                           method = distance,
+                                                           dots = dots))
 
                                       dd
 
