@@ -9,17 +9,15 @@ context("Test parallel")
 test_that("Parallel computation gives the same results as sequential", {
      skip_on_cran()
 
+     if (getOption("skip_par_tests", FALSE))
+          skip("Parallel tests disabled explicitly.")
+
      cat("\n")
 
      require(doParallel)
 
-     if (.Platform$OS.type == "windows")
-          cl <- makeCluster(detectCores())
-     else
-          cl <- makeCluster(detectCores(), "FORK")
-
+     cl <- makeCluster(detectCores())
      invisible(clusterEvalQ(cl, library(dtwclust)))
-
      registerDoParallel(cl)
 
      ## Filter excludes files that have "parallel" in them, otherwise it would be recursive
@@ -28,4 +26,20 @@ test_that("Parallel computation gives the same results as sequential", {
      stopCluster(cl)
      stopImplicitCluster()
      registerDoSEQ()
+
+     ## Also test FORK in Unix
+     if (.Platform$OS.type != "windows") {
+          cat("Test FORKs:\n")
+
+          rm(cl)
+          cl <- makeCluster(detectCores(), "FORK")
+          registerDoParallel(cl)
+
+          ## Filter excludes files that have "parallel" in them, otherwise it would be recursive
+          test_dir("./", filter = "^(?!.*parallel).*$", perl = TRUE)
+
+          stopCluster(cl)
+          stopImplicitCluster()
+          registerDoSEQ()
+     }
 })
